/*
  Tutanchamun, an arcade game
  Copyright (C) 1998  David Kastrup
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
/* xstal contains the starting position left of a shot.  That is, this
   is the location from where a current line (if there is one)
   extends.  xstpl contains the stopping position left of a shot.
   That is, this is the location after which there will no longer be a
   shot drawn.  To the right, xstar contains the starting position of
   a shot right, that is, the position up to which a shot extends.
   xstpl contains the ending position of a shot, the position up to
   which the shot has already been drawn.
   */

#include "tutdisp.h"

int hitl, phasl, yshol, xstal, xstpl,
  hitr, phasr, yshor, xstar, xstpr;

int fired;

void askfir()
{
  if (!d_fireky()) {
    fired = 0;
    return;
  }
  if (fired)
    return;
  fired = 1;
  if (timtab[SHLBYT].timebyte != DEAD_TIMER)
    timtab[SHLBYT].timebyte = 0;
  if (timtab[SHRBYT].timebyte != DEAD_TIMER)
    timtab[SHRBYT].timebyte = 0;
}

void drawshot(unsigned x, unsigned y, int w, int colour)
{
  d_drawshot(x-scrx,y,w,colour);
}

void shotl(int *p)
{
  if (!fired) {
    *p = -1;
    return;
  }
  *p = DEAD_TIMER;
  hitl = 0;
  phasl = 2;
  timtab[SMLBYT].timebyte = 0;
  yshol = mymcb.ycoor + 2;
  xstal = xstpl = mymcb.xcoor+1;
}

void shotr(int *p)
{
  if (!fired) {
    *p = -1;
    return;
  }
  *p = DEAD_TIMER;
  hitr = 0;
  phasr = 2;
  timtab[SMRBYT].timebyte = 0;
  yshor = mymcb.ycoor + 2;
  xstar = xstpr = mymcb.xcoor+3;
}

void shoml(int *p)
{
  int i;
  mcb *mp;

  *p = -1;
  if (!hitl) { /* sm02 with xstal in A */
    if (xstal >= scrx+4) { /* sm01 with scrx in A */
      if (playfield[player].staga[(xstal-4)>>2][yshol>>2] & 0x1f) {
	if (xstal&3) {
	  drawshot(xstal&~3, yshol, xstal&3, 7);
	  xstal &= ~3;
	}
	hitl = 1;
	if (phasl)
	  phasl += 3;
      } else {
	drawshot(xstal-4, yshol, 4, 7);
	xstal -= 4;
      }
    } else {
      if (xstal > scrx)
	drawshot(scrx, yshol, xstal-scrx, 7);
      xstal = scrx;
      hitl = 1;
      if (phasl)
	phasl += 3;
    }
  } else if (xstal < scrx)
    xstal = scrx;
  for (mp=mymcb.next, i=0; i<nmons; ++i, mp = mp->next) {
    if (!(mp->flag & ((1<<DEATH)|(1<<BIRTH)))
	&& yshol >  mp->ycoor && yshol < mp->ycoor + 4
	&& xstal < mp->xcoor + 4
	&& xstpl > mp->xcoor)
      {
	killim(mp);
	hitl = 1;
      }
  }
  if (phasl) {
    --phasl;
    return;
  }
  if (xstpl < xstal+4) {
    timtab[SHLBYT].timebyte = timtab[TIMBYT].timebyte != DEAD_TIMER ? -5 : DEAD_TIMER;
    timtab[SMLBYT].timebyte = DEAD_TIMER;
    if (xstpl > xstal)
      drawshot(xstal, yshol, xstpl-xstal, colbak);
    if (xstal < mymcb.xcoor+4
	&& xstpl > mymcb.xcoor)
      throwi(&mymcb);
  } else {
    xstpl -= 4;
    drawshot(xstpl, yshol, 4, colbak);
    if (xstpl < mymcb.xcoor+4
	&& xstpl+4 > mymcb.xcoor)
      throwi(&mymcb);
  }
}

void shomr(int *p)
{
  int i;
  mcb *mp;

  *p = -1;
  if (!hitr) { /* sm02 with xstal in A */
    if (xstar <= scrx+44) { /* sm01 with scrx in A */
      if (playfield[player].staga[(xstar+3)>>2][yshor>>2] & 0x1f) {
	if (xstar & 3) {
	  drawshot(xstar, yshor, ((xstar+3)&~3) - xstar, 7);
	  xstar = (xstar+3) & ~3;
	}
	hitr = 1;
	if (phasr)
	  phasr += 3;
      } else {
	drawshot(xstar, yshor, 4, 7);
	xstar += 4;
      }
    } else {
      if (scrx+48 > xstar)
	drawshot(xstar, yshor, scrx+48-xstar, 7);
      xstar = scrx+48;
      hitr = 1;
      if (phasr)
	phasr += 3;
    }
  } else if (xstar > scrx+48)
    xstar = scrx+48;
  for (mp=mymcb.next, i=0; i<nmons; ++i, mp = mp->next) {
    if (!(mp->flag & ((1<<DEATH)|(1<<BIRTH)))
	&& yshor >  mp->ycoor && yshor < mp->ycoor + 4
	&& xstar > mp->xcoor
	&& xstpr < mp->xcoor+4)
      {
	killim(mp);
	hitr = 1;
      }
  }
  if (phasr) {
    --phasr;
    return;
  }
  if (xstpr+4 > xstar) {
    timtab[SHRBYT].timebyte = timtab[TIMBYT].timebyte != DEAD_TIMER ? -3 : DEAD_TIMER;
    timtab[SMRBYT].timebyte = DEAD_TIMER;
    if (xstpr < xstar)
      drawshot(xstpr, yshor, xstar-xstpr, colbak);
    if (xstpr < mymcb.xcoor+4
	&& xstar > mymcb.xcoor)
      throwi(&mymcb);
  } else {
    drawshot(xstpr, yshor, 4, colbak);
    if (xstpr < mymcb.xcoor+4
	&& xstpr+4 > mymcb.xcoor)
      throwi(&mymcb);
    xstpr += 4;
  }
}
